C                                                                              
C ********************************************************************         
C *         S                                                        *         
C *         !                                                        *         
C *         !                                                        *         
C * F1-->---!---<---F2    SUBROUTINE FFSCCN(-FFSCT)                  *         
C *                                                                  *         
C *     Purpose: to calculate vertex amplitudes for scalar boson-    *         
C *              massive fermions vertex.                            *         
C *                                                                  *         
C *      -          C             C                                  *         
C *      U(II,AAM,Q) (AL*PL+AR*PR)  U(I,AM,P)                        *         
C *                                                                  *         
C *                                                                  *         
C *                                                                  *         
C *     I,II=: indices to specify fermion(I,II=3) or antifermion     *         
C *            (I,II=1) state.                                       *         
C *     AM,AAM=: masses of fermions.                                 *         
C *     AL,AR=: coupling constants for vertex.                       *         
C *     C,CC=: phase factors for massive fermions.                   *         
C *     P1,P2,Q1,Q2=: light-like vectors decomposed by subroutine    *         
C *                   SPLTQ.                                         *         
C *     AALL=: calculated results of vertex amplitudes for all       *         
C *            possible helicity states.                             *         
C *                                                                  *         
C *                                                                  *         
C *                                      written by H. Tanaka        *         
C ********************************************************************         
C                                                                              
C                                                                              
C       ===========================================================            
        SUBROUTINE FFSCCN(II,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,AALL)              
C       ===========================================================            
C                                                                              
        IMPLICIT REAL*8(A-H,O-Z)                                               
        COMPLEX*16 AALL,C,CC                                             
        DIMENSION P1(4),P2(4),Q1(4),Q2(4),C(2),CC(2)                           
        DIMENSION AALL(2,2)                                         
                                                                               
        I2 = II - 2                                                            
        I2 = -I2                                                               
        I2 = I2 + 2                                                            
                                                                               
        CALL FFSNNN(I2,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,AALL)                      
                                                                               
                                                                               
        RETURN                                                                 
        END                                                                    
