C
C      ************************************************************
C      *                                                          *
C      *                                                          *
C      *                                                          *
C      *                                                          *
C      *                                                          *
C      *                                                          *
C      *             ============================                 *
C      *                CHANEL   (version. 2.X)                   *
C      *             ============================                 *
C      *                                                          *
C      *                                                          *
C      *             SUBROUTINES for SUSY processes.              *
C      *             Version. 2 should be added.                  *
C      *                                                          *
C      *                   Written by H. Tanaka                   *
C      *                                                          *
C      *                                                          *
C      *                        ---------                         *
C      *                        Reference                         *
C      *                        ---------                         *
C      *    ' Numerical Calculation of Helicity Amplitudes        *
C      *      for Processes Involving Massive Fermions'           *
C      *                                                          *
C      *      H. Tanaka, Hiroshima University preprint            *
C      *         Comput. Phys. Commun. 58(1990) 153 - 168         *
C      *                                                          *
C      *                                                          *
C      *                         and                              *
C      *                                                          *
C      *    ' Numerical Calculation of Feynman Amplitudes         *
C      *      for Electroweak Theories and Application to         *
C      *      e(-) e(+) --> W(+) W(-) photon '                    *
C      *                                                          *
C      *                                                          *
C      *          H. Tanaka, T. Kanako and Y. Shimizu             *
C      *          KEK preprint 90 - 29, May 1990.                 *
C      *                                                          *
C      *                                                          *
C      *       New version according to T.Kaneko                  *
C      *                                                          *
C      *   --- TEST VERSION OF CHANSC ---                         *
C      ************************************************************
C********************************************************************
C *         S                                                        *
C *         |                                                        *
C *         |                                                        *
C * F1--<---|---<---F2    SUBROUTINE FFSNNN(FFS)                     *
C *                                                                  *
C *     Purpose: to calculate vertex amplitudes for scalar boson-    *
C *              massive fermions vertex.                            *
C *                                                                  *
C *     I,II=: indices to specify fermion(I,II=3) or antifermion     *
C *            (I,II=1) state.                                       *
C *     AM,AAM=: masses of fermions.                                 *
C *     AL,AR=: coupling constants for vertex.                       *
C *     C,CC=: phase factors for massive fermions.                   *
C *     P1,P2,Q1,Q2=: light-like vectors decomposed by subroutine    *
C *                   SPLTQ.                                         *
C *     AALL=: calculated results of vertex amplitudes for all       *
C *            possible helicity states.                             *
C *                                                                  *
C *                                                                  *
C *                                      written by H. Tanaka        *
C ********************************************************************
C

C       ===========================================================
        SUBROUTINE FFSNNN(II,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,AALL)
C       ===========================================================
C
         IMPLICIT REAL*8(A-H,O-Z)
         COMPLEX*16 AALL
         COMPLEX*16 C,CC,AALL11,AALL22,AALL12,AALL21
         DIMENSION P1(4),P2(4),Q1(4),Q2(4),C(2),CC(2)
         DIMENSION AALL(2,2),AALL11(2),AALL22(2),AALL12(2),AALL21(2)
         CALL FFS0(Q1,P1,AALL11)
         CALL FFS0(Q2,P2,AALL22)
         CALL FFS0(Q1,P2,AALL12)
         CALL FFS0(Q2,P1,AALL21)

         R  = DFLOAT(I  - 2)
         RR = DFLOAT(II - 2)

         IF (AM.LE.0.0D0) THEN
         R = 0.0D0
         ENDIF
         IF (AAM.LE.0.0D0) THEN
         RR = 0.0D0
         ENDIF

         J1 = (5-II)/2
         J2 = 3 - J1
         K1 = (5-I)/2
         K2 = 3 - K1

         AALL(J1,K2)=(AR*AALL11(1))
     &               +(AL*R*RR*C(1)*CC(2)*AALL22(2))
         AALL(J2,K1)=(AL*AALL11(2))
     &               +(AR*R*RR*C(2)*CC(1)*AALL22(1))
         AALL(J2,K2)=-(AR*RR*CC(1)*AALL21(1))
     &                -(AL*R*C(1)*AALL12(2))
         AALL(J1,K1)=-(AL*RR*CC(2)*AALL21(2))
     &                -(AR*R*C(2)*AALL12(1))
CX         AALL(J1,K2)=CC(1)*C(2)*AALL(J1,K2) 
CX         AALL(J2,K1)=CC(2)*C(1)*AALL(J2,K1)
CX         AALL(J2,K2)=CC(2)*C(2)*AALL(J2,K2)
CX         AALL(J1,K1)=CC(1)*C(1)*AALL(J1,K1)


         RETURN
         END
