C                                                                              
C                                                                              
C      ************************************************************            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *             ============================                 *            
C      *                CHANEL   (version. 4)                     *            
C      *             ============================                 *            
C      *                                                          *            
C      *               CHANEL FOR SUPER-GRAVITY                   *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *                   Written by H. Tanaka                   *            
C      *                                                          *            
C      *                                                          *            
C      *                        ---------                         *            
C      *                        Reference                         *            
C      *                        ---------                         *            
C      *    ' Numerical Calculation of Helicity Amplitudes        *            
C      *      for Processes Involving Massive Fermions'           *            
C      *                                                          *            
C      *      H. Tanaka, Hiroshima University preprint            *            
C      *         Comput. Phys. Commun. 58(1990) 153 - 168         *            
C      *                                                          *            
C      *                                                          *            
C      *                         and                              *            
C      *                                                          *            
C      *    ' Numerical Calculation of Feynman Amplitudes         *            
C      *      for Electroweak Theories and Application to         *            
C      *      e(-) e(+) --> W(+) W(-) photon '                    *            
C      *                                                          *            
C      *                                                          *            
C      *          H. Tanaka, T. Kanako and Y. Shimizu             *            
C      *          KEK preprint 90 - 29, May 1990.                 *            
C      *          Submitted to Comput. Phys. Commun.              *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      *                                                          *            
C      ************************************************************            
C                                                                              
C ********************************************************************         
C *                                                                  *         
C *                                                                  *         
C *                                                                  *         
C *   SUBROUTINE FFTT(L:I*4, II:I*4, I:I*4, AAM:R*8, AM:R*8,         *         
C *  &               AL:R*8, AR:R*8, CC:C*16(2), C:C*16(2),          *         
C *  &               Q1:R*8(4), Q2:R*8(4), P1:R*8(4), P2:R*8(4),     *         
C *  &               V1:R*8(4), V2:R*8(4), AALL:C*16(4,2,2))         *         
C *                                                                  *         
C *     Purpose: to calculate vertex amplitudes for tensor -         *         
C *              massive fermions vertex.                            *         
C *                                                                  *         
C *     L=: polarization state of vector boson.                      *         
C *     I,II=: indices to specify fermion(I,II=3) or antifermion     *         
C *            (I,II=1) state.                                       *         
C *     AM,AAM=: masses of fermions.                                 *         
C *     AL,AR=: coupling constants for vertex.                       *         
C *     C,CC=: phase factors for massive fermions.                   *         
C *     P1,P2,Q1,Q2=: light-like vectors decomposed by subroutine    *         
C *                   SPLTQ.                                         *         
C *     Q=: polarization vectors of vector bosons.                   *         
C *     AALL=: calculated results of vertex amplitudes for all       *         
C *            possible helicity states.                             *         
C *                                                                  *         
C *                                                                  *         
C *                                      written by H. Tanaka        *         
C *                                                                  *         
C *    Q1, Q2 : FOR UB OR VB                                         *         
C *    P1, P2 : FOR U OR V                                           *         
C *                                                                  *         
C *                                                                  *         
C *       -                                                          *         
C *       U[(V1V2-V2V1)/2](AL*PL+AR*PR)U                             *         
C *                                                                  *         
C *                                                                  *         
C ********************************************************************         
C                                                                              
C                                                                              
C       ===========================================================            
       SUBROUTINE FFTT(L,II,I,AAM,AM,AL,AR,CC,C,
     &                                  Q1,Q2,P1,P2,V1,V2,AALL)        
C       ===========================================================            
C                                                                              
C                                                                              
        IMPLICIT REAL*8(A-H,O-Z)                                               
        COMPLEX*16 C,CC,AALL1,AALL2,AALL                            
        DIMENSION P1(4),P2(4),Q1(4),Q2(4),C(2),CC(2),V1(4),V2(4)               
        DIMENSION AALL(4,2,2),AALL1(4,2,2),AALL2(2,2)                        
                      
       V1V2=V1(4)*V2(4)-V1(1)*V2(1)-V1(2)*V2(2)-V1(3)*V2(3)

CX     WRITE(1,*) ' ---- FFTT ------- '                                       
CX     WRITE(1,*) ' L ',L                                       
CX     WRITE(1,*) ' V1V2 ',V1V2                                       
CX      WRITE(1,*) ' AL  AR     ',AL,AR                                       
CX      WRITE(1,*) ' AM1 AM2    ',AAM,AM                                       
CX      WRITE(1,*) '   C        ',CC(1),CC(2)                                  
CX      WRITE(1,*) '   CC       ',C(1),C(2)                                    
CX      WRITE(1,*) '   P1 ',Q1(1),Q1(2),Q1(3),Q1(4)                            
CX      WRITE(1,*) '   P2 ',Q2(1),Q2(2),Q2(3),Q2(4)                            
CX      WRITE(1,*) '   Q1  ',P1(1),P1(2),P1(3),P1(4)                           
CX      WRITE(1,*) '   Q2  ',P2(1),P2(2),P2(3),P2(4)                           
CX      WRITE(1,*) '   V1  ',V1(1),V1(2),V1(3),V1(4)                           
CX      WRITE(1,*) '   V2  ',V2(1),V2(2),V2(3),V2(4)                          

        CALL FFTT2(L,II,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,V1,V2,AALL1)        
        CALL FFS(II,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,AALL2)        

       DO 10 JJ=1,2                                                            
       DO 10 J =1,2                                                            

CX     WRITE(1,*) '  JJ  J   ',JJ,J                                            
CX     WRITE(1,*) '  AALL1  ',AALL1(L,JJ,J)                                  
CX     WRITE(1,*) '  AALL2  ',AALL2(JJ,J)                                    

       AALL(L,JJ,J)=AALL1(L,JJ,J) - V1V2*AALL2(JJ,J)                                                                                                           
CX      WRITE(1,*) ' JJ J  FFTT ',JJ,J ,AALL(L,JJ,J)                           

  10    CONTINUE                                                               

                                                                               
        RETURN                                                                 
        END                                                                    
C ********************************************************************         
C *                                                                  *         
C *                                                                  *         
C *                                                                  *         
C *   SUBROUTINE FFTT2(L:i*4, II:I*4, I:I*4, AAM:R*8, AM:R*8,        *         
C *  &               AL:R*8, AR:R*8, CC:C*16(2), C:C*16(2),          *         
C *  &               Q1:R*8(4), Q2:R*8(4), P1:R*8(4), P2:R*8(4),     *         
C *  &               V1:R*8(4),V2:R*8(4), AALL:C*16(4,2,2))          *         
C *                                                                  *         
C *     Purpose: to calculate vertex amplitudes for two vectors      *
C *     and  massive fermions vertex.                                *         
C *                                                                  *         
C *     L=: polarization state of vector boson.                      *         
C *     I,II=: indices to specify fermion(I,II=3) or antifermion     *         
C *            (I,II=1) state.                                       *         
C *     AM,AAM=: masses of fermions.                                 *         
C *     AL,AR=: coupling constants for vertex.                       *         
C *     C,CC=: phase factors for massive fermions.                   *         
C *     P1,P2,Q1,Q2=: light-like vectors decomposed by subroutine    *         
C *                   SPLTQ.                                         *         
C *     V1,V2=: polarization vectors of vector bosons.               *         
C *     AALL=: calculated results of vertex amplitudes for all       *         
C *            possible helicity states.                             *         
C *                                                                  *         
C *                                                                  *         
C *                                      written by H. Tanaka        *         
C *                                                                  *         
C *    Q1, Q2 : FOR UB OR VB                                         *         
C *    P1, P2 : FOR U OR V                                           *         
C *                                                                  *         
C *                                                                  *         
C ********************************************************************         
C                                                                              
C                                                                              
C       ===========================================================            
       SUBROUTINE FFTT2(L,II,I,AAM,AM,AL,AR,CC,C,Q1,Q2,P1,P2,V1,V2,AALL)       
C       ===========================================================            
C                                                                              
C                                                                              
        IMPLICIT REAL*8(A-H,O-Z)                                               
        COMPLEX*16 AALL,C21,CC21,C22,CC22                                      
        COMPLEX*16 C,CC,AALL11,AALL22,AALL12,AALL21                            
        DIMENSION P1(4),P2(4),Q1(4),Q2(4),C(2),CC(2),V1(4),V2(4)               
        DIMENSION V21(4),V22(4),P0(4),C21(2),CC21(2),C22(2),CC22(2)            
        DIMENSION AALL(4,2,2),AALL11(4,2,2),AALL12(2,2)                        
        DIMENSION AALL21(4,2,2),AALL22(2,2)                                    
        DATA P0 / 0.D0,0.D0,0.D0,0.D0 /                                        
                                                                               
        A0 = 0.D0                                                              
        AE = 1.D0                                                              
        CALL SPLT(A0,V2,S1,S2,V21,V22)                                         
        CALL PHASEQ(1,V21,C21)                                                 
        CALL PHASEQ(2,V21,CC21)                                                
        CALL PHASEQ(1,V22,C22)                                                 
        CALL PHASEQ(2,V22,CC22)
                                                
CX      WRITE(1,*) '   V1  ',V1(1),V1(2),V1(3),V1(4)                           
CX      WRITE(1,*) '   V2  ',V2(1),V2(2),V2(3),V2(4)                           
CX      WRITE(1,*) '   V21 ',V21(1),V21(2),V21(3),V21(4)                     
CX      WRITE(1,*) '   V22 ',V22(1),V22(2),V22(3),V22(4)         
                                                                               
        CALL FFV(L,II,3,AAM,A0,AE,AE,CC,C21,Q1,Q2,V21,P0,V1,AALL11)            
        CALL FFS(3,I,A0,AM,AL,AR,CC21,C,V21,P0,P1,P2,AALL12)                   
        CALL FFV(L,II,3,AAM,A0,AE,AE,CC,C22,Q1,Q2,V22,P0,V1,AALL21)            
        CALL FFS(3,I,A0,AM,AL,AR,CC22,C,V22,P0,P1,P2,AALL22)                   
                                                                               
                                                                               
       DO 10 JJ=1,2                                                            
       DO 10 J =1,2                                                            
CX     WRITE(1,*) '  JJ  J   ',JJ,J                                            
CX     WRITE(1,*) '  AALL11  ',AALL11(L,JJ,J)                                  
CX     WRITE(1,*) '  AALL12  ',AALL12(JJ,J)                                    
CX     WRITE(1,*) '  AALL21  ',AALL21(L,JJ,J)                                  
CX     WRITE(1,*) '  AALL22  ',AALL22(JJ,J)                                    
       AALL(L,JJ,J)=(AALL11(L,JJ,1)*AALL12(1,J)                                
     &                   +AALL11(L,JJ,2)*AALL12(2,J))*S1                       
     &     +(AALL21(L,JJ,1)*AALL22(1,J)                                        
     &                   +AALL21(L,JJ,2)*AALL22(2,J))*S2                       
                                                                               
CX      WRITE(1,*) ' JJ J  FFT2 ',JJ,J ,AALL(L,JJ,J)                          
  10    CONTINUE                                                               
CX      WRITE(1,*) ' AM1 AM2    ',AAM,AM                                       
CX      WRITE(1,*) '   C        ',CC(1),CC(2)                                  
CX      WRITE(1,*) '   CC       ',C(1),C(2)                                    
CX      WRITE(1,*) '   P1 ',Q1(1),Q1(2),Q1(3),Q1(4)                            
CX      WRITE(1,*) '   P2 ',Q2(1),Q2(2),Q2(3),Q2(4)                            
CX      WRITE(1,*) '   Q1  ',P1(1),P1(2),P1(3),P1(4)                           
CX      WRITE(1,*) '   Q2  ',P2(1),P2(2),P2(3),P2(4)                           
                                                                               
        RETURN                                                                 
        END                                                                    
