C===================================================================
C Kinematics type : H2201
C                                               / 3
C TAU ...GPRI(1) <== XRUN(1)                   /
C YST ...GPRI(2) <== XRUN(2)                  / (COS3,PHI3)
C COS3...GVAL(1) <== XRUN(3)       1 ------->*<------- 2
C PHI3...GVAL(2) <== XRUN(4)       (TAU,YST)/
C PDF ...GPRI(10)<== XRUN(5)               /
C                                         / 4
C===================================================================
      SUBROUTINE KVH2201(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,COS3,COS3MN,COS3MX,SIN3
      DOUBLE PRECISION E3,P3,E3MIN,E4MIN

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0

C...Set tau.
      E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
      E4MIN = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
      TAUMN = (E3MIN+E4MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Cos3.
      E3 = (GPRI(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/GPRI(4)
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
      COS3MN = -SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      COS3MX =  SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      GCOS3(2) = MAX(-1.0D0,COS3MN)              ! Minimum Cos3.
      GCOS3(INT(GCOS3(1))+2) = MIN(1.0D0,COS3MX) ! Maximum Cos3.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS3, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(1) = COS3
      AJACOB = AJACOB*AFACTOR

C...Set Phi3. 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Pt.
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      GVAL(3) = P3*SIN3

C...Set Flavor.
      IF (NDIM.GT.4.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(5)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------
C Set 2-body kinematics.
C------------------------
      SUBROUTINE KSH2201(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER I
      DOUBLE PRECISION FACT
      DOUBLE PRECISION E1,E2,E3,P3,COS3,SIN3,PHI3,E4,Z1,Z2
      DOUBLE PRECISION PF4(4,10),AJACOB

C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.4) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass.
      IF (GPRI(3).LT.(AMASS1(IPX(3))+AMASS1(IPX(4)))**2) GO TO 999
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Set particle 1.
      PF4(1,1) = 0.0D0
      PF4(2,1) = 0.0D0
      PF4(3,1) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,1) = E1
 
C...Set particle 2.
      PF4(1,2) = 0.0D0
      PF4(2,2) = 0.0D0
      PF4(3,2) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,2) = E2
 
C...Set particle 3.
      E3 = (GPRI(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/GPRI(4)
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
      COS3 = GVAL(1)
      SIN3 = SQRT((1.0D0 - COS3)*(1.0D0 + COS3))
      PHI3 = GVAL(2)
 
      PF4(1,3) = P3*SIN3*COS(PHI3)
      PF4(2,3) = P3*SIN3*SIN(PHI3)
      PF4(3,3) = P3*COS3
      PF4(4,3) = E3
 
C...Set particle 4.
      E4 = (GPRI(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.D0/GPRI(4)
      PF4(1,4) = -PF4(1,3)
      PF4(2,4) = -PF4(2,3)
      PF4(3,4) = -PF4(3,3)
      PF4(4,4) = E4

      PQGRC(1,1) = 0.0d0
      PQGRC(2,1) = 0.0d0
      PQGRC(3,1) = PF4(3,1)+PF4(3,2)
      PQGRC(4,1) = PF4(4,1)+PF4(4,2)

C...Check kinematics. 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) IREJ = 3
      ENDDO

C...Set Jacobian.  
      AJACOB = FACT/32.D0/PI2
     $     *BETA(AMASS2(IPX(3))/GPRI(3),AMASS2(IPX(4))/GPRI(3))
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
