C===================================================================
C Kinematics type : H2303
C                                                (COS4,PHI4)
C TAU  ...GPRI(1) <== XRUN(1)       1 ------->*-------- 4
C YST  ...GPRI(2) <== XRUN(2)      (TAU,YST)   \
C E3   ...GVAL(1) <== XRUN(3)                   \    E3
C E4-E5...GVAL(2) <== XRUN(4)            E4-E5   *----- 3
C COS4 ...GVAL(3) <== XRUN(5)                   /
C COS5 ...GVAL(4) <== XRUN(6)                  /    COS5
C PHI4 ...GVAL(5) <== XRUN(7)       2 ------->*-------- 5
C PDF  ...GPRI(10)<== XRUN(8)
C===================================================================
      SUBROUTINE KVH2303(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,E3,P3,E3MIN,E3MAX
      DOUBLE PRECISION E4,P4,E4MIN,COS4,SIN4,E5,P5,E5MIN,COS5
      DOUBLE PRECISION SUME45,QESMIN,QESMAX,XCP
      DOUBLE PRECISION COS4MIN1,COS4MIN2,COS4MIN
      DOUBLE PRECISION COS4MAX1,COS4MAX2,COS4MAX
      DOUBLE PRECISION COS5MIN1,COS5MIN2,COS5MIN3,COS5MIN4,COS5MIN
      DOUBLE PRECISION COS5MAX1,COS5MAX2,COS5MAX3,COS5MAX4,COS5MAX

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0

C...Set tau.
      E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
      E4MIN = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
      E5MIN = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
      TAUMN = (E3MIN+E4MIN+E5MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set E3.
      E3MAX = (GPRI(3) + AMASS2(IPX(3)))/2.0D0/GPRI(4)
      IF (E3MIN.GT.E3MAX) GO TO 999
      E3 = E3MIN*(E3MAX/E3MIN)**XRUN(3)
      AJACOB = AJACOB*GVAL(1)*LOG(E3MAX/E3MIN)
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      GVAL(1) = E3
      P3 = SQRT(E3**2-AMASS2(IPX(3)))
 
C...Set E4-E5.
      SUME45 = GPRI(4) - E3
      QESMIN = -SUME45 + 2.0D0*E4MIN
      QESMAX =  SUME45 - 2.0D0*E5MIN
      IF (QESMIN.GT.QESMAX) GO TO 999 
      GVAL(2) = QESMIN + (QESMAX - QESMIN)*XRUN(4)
      AJACOB = AJACOB*(QESMAX - QESMIN)

      E4 = (SUME45 + GVAL(2))/2.0D0
      E5 = (SUME45 - GVAL(2))/2.0D0
      IF (E4.LT.AMASS1(IPX(4))) GO TO 999
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      P4 = SQRT(E4**2 - AMASS2(IPX(4)))
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
C...Check allowed phase space.
      IF (P4.LT.GRCPTCUT(IPX(4)-2)) GO TO 999
      IF (P5.LT.GRCPTCUT(IPX(5)-2)) GO TO 999
      IF (P4+P5-P3.LT.0.0D0) GO TO 999
      IF (P3-P4+P5.LT.0.0D0) GO TO 999
      IF (P3+P4-P5.LT.0.0D0) GO TO 999

C...Set Phi4.
      GVAL(5) = XRUN(7)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos4.
C...Due to Pt.
      COS4MIN1 = -SQRT(P4**2-GRCPTCUT(IPX(4)-2)**2)/P4
      COS4MAX1 =  SQRT(P4**2-GRCPTCUT(IPX(4)-2)**2)/P4
C...Due to Eta.
      COS4MIN2 = -E4/P4*TANH(GRCETACUT(IPX(4)-2))
      COS4MAX2 =  E4/P4*TANH(GRCETACUT(IPX(4)-2))
 
      COS4MIN = MAX(-1.0D0,COS4MIN1,COS4MIN2)
      COS4MAX = MIN(1.0D0,COS4MAX1,COS4MAX2)
 
      RNA = XRUN(5)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $     COS4,AFACTOR,COS4MIN,COS4MAX)
      GVAL(3) = COS4
      AJACOB = AJACOB*AFACTOR
      SIN4 = SQRT((1.0D0+COS4)*(1.0D0-COS4))

C...Set Cos5.
C...Due to Pt.
      COS5MIN1 = -SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
      COS5MAX1 =  SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
C...Due to Eta.
      COS5MIN2 = -E5/P5*TANH(GRCETACUT(IPX(5)-2))
      COS5MAX2 =  E5/P5*TANH(GRCETACUT(IPX(5)-2))
C...Due to Cosine Constraint.
      COS5MIN3 = -(P3 + P4*COS4)/P5
      COS5MAX3 = -(-P3 + P4*COS4)/P5
C...Due to Phi Constraint.
      XCP = (P3**2-P4**2-P5**2)/2.0D0/P4/P5
      IF (XCP.GT.0) THEN
         COS5MIN4 = XCP*(COS4 - SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
         COS5MAX4 = XCP*(COS4 + SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
      ELSE
         COS5MIN4 = XCP*(COS4 + SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
         COS5MAX4 = XCP*(COS4 - SIN4*SQRT(1.0D0/XCP**2 - 1.0D0))
      ENDIF

      COS5MIN = MAX(-1.0D0,COS5MIN1,COS5MIN2,COS5MIN3,COS5MIN4)
      COS5MAX = MIN(1.0D0,COS5MAX1,COS5MAX2,COS5MAX3,COS5MAX4)
      IF (COS5MIN.GT.COS5MAX) GO TO 999
 
      RNA = XRUN(6)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $     COS5,AFACTOR,COS5MIN,COS5MAX)
      GVAL(4) = COS5
      AJACOB = AJACOB*AFACTOR

C...Set Flavor.
      IF (NDIM.GT.7.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(8)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------------------
C Set 3-body fusion type kinematics.
C------------------------------------
      SUBROUTINE KSH2303(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER I
      DOUBLE PRECISION FACT
      DOUBLE PRECISION E1,E2,E3,E4,E5,P1,P3,P4,P5,PT3,PT4,PT5
      DOUBLE PRECISION COS3,COS4,COS5,SIN3,SIN4,SIN5,PHI3,PHI4,PHI5
      DOUBLE PRECISION COSP34,SINP34
      DOUBLE PRECISION PF4(4,10),AJACOB
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.5) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Set particle 4.
      E4 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
      P4 = SQRT(E4**2 - AMASS2(IPX(4)))
      COS4 = GVAL(3)
      SIN4 = SQRT((1.0D0 - COS4)*(1.0D0 + COS4))
      PT4 = P4*SIN4
      PHI4 = GVAL(5)
 
      PF4(1,IPX(4)) = P4*SIN4*COS(PHI4)
      PF4(2,IPX(4)) = P4*SIN4*SIN(PHI4)
      PF4(3,IPX(4)) = P4*COS4
      PF4(4,IPX(4)) = E4
 
C...Set particle 5.
      E5 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
      COS5 = GVAL(4)
      SIN5 = SQRT((1.0D0 - COS5)*(1.0D0 + COS5))
      PT5 = P5*SIN5

C...Set particle 3.
      E3 = GVAL(1)
      P3 = SQRT((E3 - AMASS1(IPX(3)))*(E3 + AMASS1(IPX(3))))
      COS3 = (P5*COS5 + P4*COS4)/P3
      SIN3 = SQRT((1.0D0 - COS3)*(1.0D0 + COS3))
      PT3 = P3*SIN3
      PHI5 = PHI4 + ACOS((PT3**2-PT4**2-PT5**2)/2.0D0/PT4/PT5)

      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5
 
      PF4(1,IPX(3)) = -PF4(1,IPX(4)) - PF4(1,IPX(5))
      PF4(2,IPX(3)) = -PF4(2,IPX(4)) - PF4(2,IPX(5))
      PF4(3,IPX(3)) = -PF4(3,IPX(4)) - PF4(3,IPX(5))
      PF4(4,IPX(3)) = GVAL(1)

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/8.0D0/(2.0D0*PI)**5/SIN4/SIN5
     $     /ABS(SIN(ACOS((PT3**2-PT4**2-PT5**2)/2.0D0/PT4/PT5)))
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
