C===================================================================
C Kinematics type : H2403
C                                             3     4
C TAU  ...GPRI(1) <== XRUN(1)      (COS3,PHI3) \   /
C YST  ...GPRI(2) <== XRUN(2)                   \ /
C COS34...GVAL(1) <== XRUN(3)                    *
C PHI34...GVAL(2) <== XRUN(4)                   / (Q34,COS34,PHI34)
C Q34  ...GVAL(3) <== XRUN(5)       (TAU,YST)  /    (resonance)
C Q56  ...GVAL(4) <== XRUN(6)       1 ------->*<------- 2 
C COS3 ...GVAL(5) <== XRUN(7)                /
C PHI3 ...GVAL(6) <== XRUN(8)           Q56 /  (resonance)
C COS5 ...GVAL(7) <== XRUN(9)              *
C PHI5 ...GVAL(8) <== XRUN(10)            / \
C PDF  ...GPRI(10)<== XRUN(11)          5/  6\ (COS5,PHI5)
C===================================================================
      SUBROUTINE KVH2403(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,COS34,COS3,COS5
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,E6MIN
      DOUBLE PRECISION P3MIN,P4MIN,P5MIN,P6MIN
      DOUBLE PRECISION Q34,Q34MN,Q34MX,R34MN
      DOUBLE PRECISION Q56,Q56MN,Q56MX,R56MN

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16
      IF (GQ22(INT(GQ22(1))+2).LT.1.0D-1) GQ22(INT(GQ22(1))+2)=1.0D16

C...Set Q34.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      R34MN = MIN(GRCRCONCUT(IPX(3)-2),GRCRCONCUT(IPX(4)-2))
      Q34MN = AMASS2(IPX(3))+AMASS2(IPX(4))+2.0D0*E3MIN*E4MIN
     $     -2.0D0*P3MIN*P4MIN*COS(R34MN)
      Q34MX = (GRCECM-E5MIN-E6MIN)**2
      GQ12(2) = MAX(GQ12(2),Q34MN)                           ! Minimum Q34.
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q34MX) ! Maximum Q34.
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(5)
      CALL GRCSQ12(RNA, Q34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(3) = Q34
      AJACOB = AJACOB*AFACTOR

C...Set Q56.
      R56MN = MIN(GRCRCONCUT(IPX(5)-2),GRCRCONCUT(IPX(6)-2))
      Q56MN = AMASS2(IPX(5))+AMASS2(IPX(6))+2.0D0*E5MIN*E6MIN
     $     -2.0D0*P5MIN*P6MIN*COS(R56MN)
CCC      Q56MX = (GRCECM-E3MIN-E4MIN)**2
      Q56MX = (GRCECM-SQRT(Q34))**2
      GQ22(2) = MAX(GQ22(2),Q56MN)                           ! Minimum Q56.
      GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),Q56MX) ! Maximum Q56.
C...Now is only Z/gamma pair production.
      RNA = XRUN(6)
      IF (RNA.LT.GQ22(9)) THEN
         GQ22(INT(GQ22(1))+1) = MIN(GQ22(INT(GQ22(1))+1),Q56MX)
      ELSE
         IF (Q56MX.LT.GQ22(INT(GQ22(1))+1)) GO TO 999
      ENDIF
      CALL GRCSQ22(RNA, Q56, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(4) = Q56
      AJACOB = AJACOB*AFACTOR

C...Set tau.
      TAUMN = (SQRT(Q34)+SQRT(Q56))**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Cos34, Phi34.
      GCOS3(2) = -1.0D0              ! Minimum Cos3.
      GCOS3(INT(GCOS3(1))+2) = 1.0D0 ! Maximum Cos3.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(1) = COS34
      AJACOB = AJACOB*AFACTOR
 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos3, Phi3 in 34 CM frame.
      RNA = XRUN(7)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,COS3,AFACTOR,-1.0D0,1.0D0)
      GVAL(5) = COS3
      AJACOB = AJACOB*AFACTOR

      GVAL(6) = XRUN(8)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos5, Phi5 in 56 CM frame.
      RNA = XRUN(9)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,COS5,AFACTOR,-1.0D0,1.0D0)
      GVAL(7) = COS5
      AJACOB = AJACOB*AFACTOR

      GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI

C...Set Flavor.
      IF (NDIM.GT.10.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(11)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------
C Set 4-body kinematics.
C------------------------
      SUBROUTINE KSH2403(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT,Z1,Z2
      DOUBLE PRECISION E1,E2,E34,P34,E56,COS34,SIN34,PHI34
      DOUBLE PRECISION E3,P3,COS3,SIN3,PHI3,E4
      DOUBLE PRECISION E5,P5,COS5,SIN5,PHI5,E6
      DOUBLE PRECISION PF4(4,10),Q1(4),Q2(4),AJACOB
 
C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass.
      IF (GVAL(3).LT.(AMASS1(IPX(3))+AMASS1(IPX(4)))**2) GO TO 999
      IF (GVAL(4).LT.(AMASS1(IPX(5))+AMASS1(IPX(6)))**2) GO TO 999

C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)

C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(1))*(E1+AMASS1(1)))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(2))*(E2+AMASS1(2)))
      PF4(4,IPX(2)) = E2

C...Set particle 3 in 34 CM frame.
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.D0/SQRT(GVAL(3))
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
      COS3 = GVAL(5)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(6)

      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3

C...Set particle 4 in 34 CM frame.
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.D0/SQRT(GVAL(3))
      IF (E4.LT.AMASS1(IPX(4))) GO TO 999

      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...Set particle 5 in 56 CM frame
      E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))/2.D0/SQRT(GVAL(4))
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      P5 = SQRT((E5-AMASS1(IPX(5)))*(E5+AMASS1(IPX(5))))
      COS5 = GVAL(7)
      SIN5 = SQRT((1.0D0-COS5)*(1.0D0+COS5))
      PHI5 = GVAL(8)
 
      IF (GSPR(1).EQ.7.OR.GSPR(2).EQ.7) THEN
         IF (XRUN(9).GE.0.25.AND.XRUN(9).LT.0.75) THEN
            COS5 = -COS5
            SIN5 = -SIN5
         ENDIF
      ENDIF
 
      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5
 
C...Set particle 6 in 56 CM frame
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))/2.D0/SQRT(GVAL(4))
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999

      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = E6
 
C...Set q34 and q56.
      COS34 = GVAL(1)
      SIN34 = SQRT((1.0D0-COS34)*(1.0D0+COS34))
      PHI34 = GVAL(2)
 
      E34 = (GPRI(3) + GVAL(3) - GVAL(4))/2.D0/GPRI(4)
      P34 = SQRT(E34*E34 - GVAL(3))
      Q1(1) = P34*SIN34*COS(PHI34)
      Q1(2) = P34*SIN34*SIN(PHI34)
      Q1(3) = P34*COS34
      Q1(4) = E34

      PQGRC(1,1) = Q1(1)
      PQGRC(2,1) = Q1(2)
      PQGRC(3,1) = Q1(3)
      PQGRC(4,1) = Q1(4)
 
      E56 = (GPRI(3) + GVAL(4) - GVAL(3))/2.D0/GPRI(4)
      Q2(1) = -Q1(1)
      Q2(2) = -Q1(2)
      Q2(3) = -Q1(3)
      Q2(4) = E56

      PQGRC(1,2) = Q2(1)
      PQGRC(2,2) = Q2(2)
      PQGRC(3,2) = Q2(3)
      PQGRC(4,2) = Q2(4)

C...Boost from 34 CM frame to Lab. frame.
      CALL GRCW2LB(Q1,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(Q1,PF4(1,IPX(4)),PF4(1,IPX(4)))

C...Boost from 56 CM frame to Lab. frame.
      CALL GRCW2LB(Q2,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(Q2,PF4(1,IPX(6)),PF4(1,IPX(6)))

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/(2*PI)**2/(32*PI2)**3
     $     *BETA(GVAL(3)/GPRI(3),GVAL(4)/GPRI(3))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
