C===================================================================
C Kinematics type : H2501
C                                            3     4
C TAU  ...GPRI(1) <== XRUN(1)     (COS3,PHI3) \   /
C YST  ...GPRI(2) <== XRUN(2)                  \ /
C COS34...GVAL(1) <== XRUN(3)                   *
C PHI34...GVAL(2) <== XRUN(4)                  /
C Q34  ...GVAL(3) <== XRUN(5)     (TAU,YST)   /  (Q34,COS34,PHI34)
C Q56  ...GVAL(4) <== XRUN(6)      1 ------->*<------- 2 
C Q567 ...GVAL(5) <== XRUN(7)               /
C COS7 ...GVAL(6) <== XRUN(8)              / Q567
C PHI7 ...GVAL(7) <== XRUN(9)             *
C COS3 ...GVAL(8) <== XRUN(10)           / \ Q56
C PHI3 ...GVAL(9) <== XRUN(11)          /   *----- 5
C COS5 ...GVAL(10)<== XRUN(12)         /    |   (COS5,PHI5)
C PHI5 ...GVAL(11)<== XRUN(13)        7     |
C PDF  ...GPRI(10)<== XRUN(14)  (COS7,PHI7) 6
C===================================================================
      SUBROUTINE KVH2501(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,Q567,Q567MIN,Q567MAX
      DOUBLE PRECISION COS34,COS3,COS5,COS7
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,E6MIN,E7MIN
      DOUBLE PRECISION P3MIN,P4MIN,P5MIN,P6MIN,P7MIN
      DOUBLE PRECISION Q34,Q34MN,Q34MX,R34MN
      DOUBLE PRECISION Q56,Q56MN,Q56MX,R56MN

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16
      IF (GQ22(INT(GQ22(1))+2).LT.1.0D-1) GQ22(INT(GQ22(1))+2)=1.0D16

C...Set Q34.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      P7MIN = GRCPTCUT(IPX(7)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      E7MIN = SQRT(P7MIN**2+AMASS2(IPX(7)))
      R34MN = MIN(GRCRCONCUT(IPX(3)-2),GRCRCONCUT(IPX(4)-2))
      Q34MN = AMASS2(IPX(3))+AMASS2(IPX(4))+2.0D0*E3MIN*E4MIN
     $     -2.0D0*P3MIN*P4MIN*COS(R34MN)
      Q34MX = (GRCECM-E5MIN-E6MIN-E7MIN)**2
      GQ12(2) = MAX(GQ12(2),Q34MN)                           ! Minimum Q34
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q34MX) ! Maximum Q34
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(5)
      CALL GRCSQ12(RNA, Q34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(3) = Q34
      AJACOB = AJACOB*AFACTOR

C...Set Q56.
      R56MN = MIN(GRCRCONCUT(IPX(5)-2),GRCRCONCUT(IPX(6)-2))
      Q56MN = AMASS2(IPX(5))+AMASS2(IPX(6))+2.0D0*E5MIN*E6MIN
     $     -2.0D0*P5MIN*P6MIN*COS(R56MN)
      Q56MX = (GRCECM-E3MIN-E4MIN-E7MIN)**2
      GQ22(2) = MAX(GQ22(2),Q56MN)                           ! Minimum Q56
      GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),Q56MX) ! Maximum Q56
      IF (GQ22(2).GT.GQ22(INT(GQ22(1))+2)) GO TO 999
      RNA = XRUN(6)
      CALL GRCSQ22(RNA, Q56, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(4) = Q56
      AJACOB = AJACOB*AFACTOR

C...Set tau.
      TAUMN = (SQRT(GVAL(3))+SQRT(GVAL(4))+E7MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Q567.
      Q567MIN = (SQRT(GVAL(4))+E7MIN)**2-GRCPTCUT(IPX(7)-2)**2
      Q567MAX = (GPRI(4)-SQRT(GVAL(3)))**2
      IF (Q567MIN.GT.Q567MAX) GO TO 999

      GVAL(5) = Q567MIN*(Q567MAX/Q567MIN)**XRUN(7)
      AJACOB = AJACOB*GVAL(5)*LOG(Q567MAX/Q567MIN)

C...Set Cos34.
      GCOS3(2) = -1.0D0              ! Minimum Cos3.
      GCOS3(INT(GCOS3(1))+2) = 1.0D0 ! Maximum Cos3.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS34, AFACTOR)
      GVAL(1) = COS34
      AJACOB = AJACOB*AFACTOR

C...Set Phi34. 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos7, Phi7 in 567 CM frame.
      RNA = XRUN(8)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,COS7,AFACTOR,-1.0D0,1.0D0)
      GVAL(6) = COS7
      AJACOB = AJACOB*AFACTOR

      GVAL(7) = XRUN(9)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos3, Phi3 in 34 CM frame.
      RNA = XRUN(10)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,COS3,AFACTOR,-1.0D0,1.0D0)
      GVAL(8) = COS3
      AJACOB = AJACOB*AFACTOR

      GVAL(9) = XRUN(11)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos5, Phi5 in 56 CM frame.
      RNA = XRUN(12)
      CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,COS5,AFACTOR,-1.0D0,1.0D0)
      GVAL(10) = COS5
      AJACOB = AJACOB*AFACTOR

      GVAL(11) = XRUN(13)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Flavor.
      IF (NDIM.GT.14.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(14)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C-------------------------------------
C Set 5-body cascade type kinematics.
C-------------------------------------
      SUBROUTINE KSH2501(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT,Z1,Z2,AJACOB
      DOUBLE PRECISION E1,E2,E3,E4,E5,E6,E7,P3,P5,P7,E34,P34,E56
      DOUBLE PRECISION COS34,SIN34,PHI34,COS7,SIN7,PHI7
      DOUBLE PRECISION COS3,SIN3,PHI3,COS5,SIN5,PHI5
      DOUBLE PRECISION PF4(4,10),PQ567(4),PQ34(4),PQ56(4)

C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.7) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
         GO TO 999
      ENDIF
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      IF (E1.LT.AMASS1(IPX(1))) GO TO 999
      IF (E2.LT.AMASS1(IPX(2))) GO TO 999

C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Set Q34.
      COS34 = GVAL(1)
      SIN34 = SQRT((1.0D0-COS34)*(1.0D0+COS34))
      PHI34 = GVAL(2)
      E34 = (GPRI(3)+GVAL(3)-GVAL(5))/2.0D0/GPRI(4)
      IF (E34.LT.SQRT(GVAL(3))) GO TO 999
      P34 = SQRT(E34**2 - GVAL(3))

      PQ34(1) = P34*SIN34*COS(PHI34)
      PQ34(2) = P34*SIN34*SIN(PHI34)
      PQ34(3) = P34*COS34
      PQ34(4) = E34

      PQGRC(1,1) = PQ34(1)
      PQGRC(2,1) = PQ34(2)
      PQGRC(3,1) = PQ34(3)
      PQGRC(4,1) = PQ34(4)

C...Set particle 3 in 34 CM frame.
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/SQRT(GVAL(3))
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3 - AMASS1(IPX(3)))*(E3 + AMASS1(IPX(3))))
      COS3 = GVAL(8)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(9)
 
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3

C...Set particle 4 in 34 CM frame.
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.0D0/SQRT(GVAL(3))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...Boost from 34 CM frame to Lab. frame.
      CALL GRCW2LB(PQ34,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(PQ34,PF4(1,IPX(4)),PF4(1,IPX(4)))

C...Set particle 567.
      PQ567(1) = -PQ34(1)
      PQ567(2) = -PQ34(2)
      PQ567(3) = -PQ34(3)
      PQ567(4) = (GPRI(3)+GVAL(5)-GVAL(3))/2.0D0/GPRI(4)
      IF (PQ567(4).LT.SQRT(GVAL(5))) GO TO 999

      PQGRC(1,3) = PQ567(1)
      PQGRC(2,3) = PQ567(2)
      PQGRC(3,3) = PQ567(3)
      PQGRC(4,3) = PQ567(4)

C...Set particle 7 in 567 CM frame.
      E7 = (GVAL(5)+AMASS2(IPX(7))-GVAL(4))/2.0D0/SQRT(GVAL(5))
      IF (E7.LT.AMASS1(IPX(7))) GO TO 999
      P7 = SQRT((E7 - AMASS1(IPX(7)))*(E7 + AMASS1(IPX(7))))
      COS7 = GVAL(6)
      SIN7 = SQRT((1.0D0-COS7)*(1.0D0+COS7))
      PHI7 = GVAL(7)

      PF4(1,IPX(7)) = P7*SIN7*COS(PHI7)
      PF4(2,IPX(7)) = P7*SIN7*SIN(PHI7)
      PF4(3,IPX(7)) = P7*COS7
      PF4(4,IPX(7)) = E7

C...Set particle 56 in 567 CM frame.
      E56 = (GVAL(5)+GVAL(4)-AMASS2(IPX(7)))/2.0D0/SQRT(GVAL(5))
      IF (E56.LT.SQRT(GVAL(4))) GO TO 999

      PQ56(1) = -PF4(1,IPX(7))
      PQ56(2) = -PF4(2,IPX(7))
      PQ56(3) = -PF4(3,IPX(7))
      PQ56(4) = E56

      PQGRC(1,2) = PQ56(1)
      PQGRC(2,2) = PQ56(2)
      PQGRC(3,2) = PQ56(3)
      PQGRC(4,2) = PQ56(4)

C...Set particle 5 in 56 CM frame.
      E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))/2.0D0/SQRT(GVAL(4))
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      P5 = SQRT((E5 - AMASS1(IPX(5)))*(E5 + AMASS1(IPX(5))))
      COS5 = GVAL(10)
      SIN5 = SQRT((1.0D0-COS5)*(1.0D0+COS5))
      PHI5 = GVAL(11)

      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5

C...Set particle 6 in 56 CM frame
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))/2.0D0/SQRT(GVAL(4))
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999

      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = E6

C...Boost from 56 CM frame to 567 frame.
      CALL GRCW2LB(PQ56,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(PQ56,PF4(1,IPX(6)),PF4(1,IPX(6)))

C...Boost from 567 CM frame to Lab. frame.
      CALL GRCW2LB(PQ567,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(PQ567,PF4(1,IPX(6)),PF4(1,IPX(6)))
      CALL GRCW2LB(PQ567,PF4(1,IPX(7)),PF4(1,IPX(7)))
      CALL GRCW2LB(PQ567,PQGRC(1,2),PQGRC(1,2))

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/(2.0D0*PI)**11/8.0D0**4
     $     *BETA(GVAL(3)/GPRI(3),GVAL(5)/GPRI(3))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(AMASS2(IPX(7))/GVAL(5),GVAL(4)/GVAL(5))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
