C===================================================================
C Kinematics type : H2601
C                                                  3
C TAU   ...GPRI(1) <== XRUN(1)               7     | (COS3,PHI3)
C YST   ...GPRI(2) <== XRUN(2)                \    |
C COS347...GVAL(1) <== XRUN(3)                 \   *----- 4
C PHI347...GVAL(2) <== XRUN(4)                  \ / (Q34,COS34,PHI34)
C Q34   ...GVAL(3) <== XRUN(5)                   *
C Q56   ...GVAL(4) <== XRUN(6)                  /
C Q347  ...GVAL(5) <== XRUN(7)     (TAU,YST)   /  (Q347,COS347,PHI347)
C Q568  ...GVAL(6) <== XRUN(8)      1 ------->*<------- 2 
C COS34 ...GVAL(7) <== XRUN(9)               /
C PHI34 ...GVAL(8) <== XRUN(10)             / Q568
C COS56 ...GVAL(9) <== XRUN(11)            *
C PHI56 ...GVAL(10)<== XRUN(12)           / \ (Q56,COS56,PHI56)
C COS3  ...GVAL(11)<== XRUN(13)          /   *----- 5
C PHI3  ...GVAL(12)<== XRUN(14)         /    |   (COS5,PHI5)
C COS5  ...GVAL(13)<== XRUN(15)        8     |
C PHI5  ...GVAL(14)<== XRUN(16)              6
C PDF   ...GPRI(10)<== XRUN(17)
C===================================================================
      SUBROUTINE KVH2601(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST
      DOUBLE PRECISION COS347,COS3,COS5,COS34,COS56
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,E6MIN,E7MIN,E8MIN
      DOUBLE PRECISION P3MIN,P4MIN,P5MIN,P6MIN,P7MIN,P8MIN
      DOUBLE PRECISION Q34,Q34MN,Q34MX,R34MN
      DOUBLE PRECISION Q56,Q56MN,Q56MX,R56MN
      DOUBLE PRECISION Q347,Q347MIN,Q347MAX
      DOUBLE PRECISION Q568,Q568MIN,Q568MAX

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16
      IF (GQ22(INT(GQ22(1))+2).LT.1.0D-1) GQ22(INT(GQ22(1))+2)=1.0D16

C...Set Q34.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      P7MIN = GRCPTCUT(IPX(7)-2)
      P8MIN = GRCPTCUT(IPX(8)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      E7MIN = SQRT(P7MIN**2+AMASS2(IPX(7)))
      E8MIN = SQRT(P8MIN**2+AMASS2(IPX(8)))
      R34MN = MIN(GRCRCONCUT(IPX(3)-2),GRCRCONCUT(IPX(4)-2))
      Q34MN = AMASS2(IPX(3))+AMASS2(IPX(4))+2.0D0*E3MIN*E4MIN
     $     -2.0D0*P3MIN*P4MIN*COS(R34MN)
      Q34MX = (GRCECM-E5MIN-E6MIN-E7MIN-E8MIN)**2
      GQ12(2) = MAX(GQ12(2),Q34MN)                           ! Minimum Q34
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q34MX) ! Maximum Q34
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(5)
      CALL GRCSQ12(RNA, Q34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(3) = Q34
      AJACOB = AJACOB*AFACTOR

C...Set Q56.
      R56MN = MIN(GRCRCONCUT(IPX(5)-2),GRCRCONCUT(IPX(6)-2))
      Q56MN = AMASS2(IPX(5))+AMASS2(IPX(6))+2.0D0*E5MIN*E6MIN
     $     -2.0D0*P5MIN*P6MIN*COS(R56MN)
      Q56MX = (GRCECM-SQRT(Q34)-E7MIN-E8MIN)**2
      GQ22(2) = MAX(GQ22(2),Q56MN)                           ! Minimum Q56.
      GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),Q56MX) ! Maximum Q56.
      IF (GQ22(2).GT.GQ22(INT(GQ22(1))+2)) GO TO 999
      RNA = XRUN(6)
      CALL GRCSQ22(RNA, Q56, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(4) = Q56
      AJACOB = AJACOB*AFACTOR

C...Set Q347.
      Q347MIN = (SQRT(Q34)+E7MIN)**2-P7MIN**2
      Q347MAX = (GRCECM-SQRT(Q56)-E8MIN)**2
      IF (Q347MIN.GT.Q347MAX) GO TO 999

      GVAL(5) = Q347MIN*(Q347MAX/Q347MIN)**XRUN(7)
      AJACOB = AJACOB*GVAL(5)*LOG(Q347MAX/Q347MIN)

C...Set Q568.
      Q568MIN = (SQRT(Q56)+E8MIN)**2-P8MIN**2
      Q568MAX = (GRCECM-SQRT(GVAL(5)))**2
      IF (Q568MIN.GT.Q568MAX) GO TO 999

      GVAL(6) = Q568MIN*(Q568MAX/Q568MIN)**XRUN(8)
      AJACOB = AJACOB*GVAL(6)*LOG(Q568MAX/Q568MIN)

C...Set tau.
      TAUMN = (SQRT(GVAL(5))+SQRT(GVAL(6)))**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Cos347.
      GCOS3(2) = -1.0D0              ! Minimum Cos3.
      GCOS3(INT(GCOS3(1))+2) = 1.0D0 ! Maximum Cos3.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS347, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(1) = COS347
      AJACOB = AJACOB*AFACTOR

C...Set Phi347. 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos34, Phi34 in 347 CM frame.
      RNA = XRUN(9)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,COS34,AFACTOR,-1.0D0,1.0D0)
      GVAL(7) = COS34
      AJACOB = AJACOB*AFACTOR

      GVAL(8) = XRUN(10)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos56, Phi56 in 568 CM frame.
      RNA = XRUN(11)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,COS56,AFACTOR,-1.0D0,1.0D0)
      GVAL(9) = COS56
      AJACOB = AJACOB*AFACTOR

      GVAL(10) = XRUN(12)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos3, Phi3 in 34 CM frame.
      RNA = XRUN(13)
      CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,COS3,AFACTOR,-1.0D0,1.0D0)
      GVAL(11) = COS3
      AJACOB = AJACOB*AFACTOR

      GVAL(12) = XRUN(14)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos5, Phi5 in 56 CM frame.
      RNA = XRUN(15)
      CALL GRCSBCS(INT(GSPR(4)),GEPS(4),RNA,COS5,AFACTOR,-1.0D0,1.0D0)
      GVAL(13) = COS5
      AJACOB = AJACOB*AFACTOR

      GVAL(14) = XRUN(16)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Flavor.
      IF (NDIM.GT.16.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(17)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C-------------------------------------
C Set 6-body culster type kinematics.
C-------------------------------------
      SUBROUTINE KSH2601(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT,Z1,Z2,AJACOB
      DOUBLE PRECISION E1,E2,E347,P347,COS347,SIN347,PHI347,E568
      DOUBLE PRECISION E3,P3,COS3,SIN3,PHI3,E4
      DOUBLE PRECISION E5,P5,COS5,SIN5,PHI5,E6
      DOUBLE PRECISION E34,P34,COS34,SIN34,PHI34,E7
      DOUBLE PRECISION E56,P56,COS56,SIN56,PHI56,E8
      DOUBLE PRECISION PF4(4,10),QT1(4),QT2(4),Q34(4),Q56(4)
 
C...beta function
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)

      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.8) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
         GO TO 999
      ENDIF
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      IF (E1.LT.AMASS1(IPX(1))) GO TO 999
      IF (E2.LT.AMASS1(IPX(2))) GO TO 999

C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2

C...Set Q347.
      COS347 = GVAL(1)
      SIN347 = SQRT((1.0D0-COS347)*(1.0D0+COS347))
      PHI347 = GVAL(2)
      E347 = (GPRI(3)+GVAL(5)-GVAL(6))/2.0D0/GPRI(4)
      IF (E347.LT.SQRT(GVAL(5))) GO TO 999
      P347 = SQRT(E347**2 - GVAL(5))

      QT1(1) = P347*SIN347*COS(PHI347)
      QT1(2) = P347*SIN347*SIN(PHI347)
      QT1(3) = P347*COS347
      QT1(4) = E347

C...Set Q568.
      E568 = (GPRI(3)+GVAL(6)-GVAL(5))/2.0D0/GPRI(4)
      IF (E568.LT.SQRT(GVAL(6))) GO TO 999
      QT2(1) = -QT1(1)
      QT2(2) = -QT1(2)
      QT2(3) = -QT1(3)
      QT2(4) = E568

      PQGRC(1,1) = QT1(1)
      PQGRC(2,1) = QT1(2)
      PQGRC(3,1) = QT1(3)
      PQGRC(4,1) = QT1(4)

      PQGRC(1,2) = QT2(1)
      PQGRC(2,2) = QT2(2)
      PQGRC(3,2) = QT2(3)
      PQGRC(4,2) = QT2(4)

C...Set Q34.
      COS34 = GVAL(7)
      SIN34 = SQRT((1.0D0-COS34)*(1.0D0+COS34))
      PHI34 = GVAL(8)
      E34 = (GVAL(5)+GVAL(3)-AMASS2(IPX(7)))/2.D0/SQRT(GVAL(5))
      IF (E34.lt.SQRT(GVAL(3))) GO TO 999
      P34 = SQRT(E34**2 - GVAL(3))

      Q34(1) = P34*SIN34*COS(PHI34)
      Q34(2) = P34*SIN34*SIN(PHI34)
      Q34(3) = P34*COS34
      Q34(4) = E34

C...Set particle 7 in 347 CM frame.
      E7 = (GVAL(5)+AMASS2(IPX(7))-GVAL(3))/2.D0/SQRT(GVAL(5))
      IF (E7.lt.AMASS1(IPX(7))) GO TO 999
      PF4(1,IPX(7)) = -Q34(1)
      PF4(2,IPX(7)) = -Q34(2)
      PF4(3,IPX(7)) = -Q34(3)
      PF4(4,IPX(7)) = E7

C...Set particle 3 in 34 CM frame.
      COS3 = GVAL(11)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(12)
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.D0/SQRT(GVAL(3))
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
 
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3
 
C...Set particle 4 in 34 CM frame.
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.D0/SQRT(GVAL(3))
      IF (E4.LT.AMASS1(IPX(4))) GO TO 999
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...Boost from 34 CM frame to 347 frame.
      CALL GRCW2LB(Q34,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(Q34,PF4(1,IPX(4)),PF4(1,IPX(4)))

C...Boost from 347 CM frame to Lab. frame.
      CALL GRCW2LB(QT1,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(QT1,PF4(1,IPX(4)),PF4(1,IPX(4)))
      CALL GRCW2LB(QT1,PF4(1,IPX(7)),PF4(1,IPX(7)))
      CALL GRCW2LB(QT1,Q34,Q34)

      PQGRC(1,3) = Q34(1)
      PQGRC(2,3) = Q34(2)
      PQGRC(3,3) = Q34(3)
      PQGRC(4,3) = Q34(4)

C...Set Q56.
      COS56 = GVAL(9)
      SIN56 = SQRT((1.0D0-COS56)*(1.0D0+COS56))
      PHI56 = GVAL(10)
      E56 = (GVAL(6)+GVAL(4)-AMASS2(IPX(8)))/2.D0/SQRT(GVAL(6))
      IF (E56.lt.SQRT(GVAL(4))) GO TO 999
      P56 = SQRT(E56**2 - GVAL(4))

      Q56(1) = P56*SIN56*COS(PHI56)
      Q56(2) = P56*SIN56*SIN(PHI56)
      Q56(3) = P56*COS56
      Q56(4) = E56

C...Set particle 8 in 568 CM frame.
      E8 = (GVAL(6)+AMASS2(IPX(8))-GVAL(4))/2.D0/SQRT(GVAL(6))
      IF (E8.lt.AMASS1(IPX(8))) GO TO 999
      PF4(1,IPX(8)) = -Q56(1)
      PF4(2,IPX(8)) = -Q56(2)
      PF4(3,IPX(8)) = -Q56(3)
      PF4(4,IPX(8)) = E8

C...Set particle 5 in 56 CM frame.
      COS5 = GVAL(13)
      SIN5 = SQRT((1.0D0-COS5)*(1.0D0+COS5))
      PHI5 = GVAL(14)
      E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))/2.D0/SQRT(GVAL(4))
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      P5 = SQRT((E5-AMASS1(IPX(5)))*(E5+AMASS1(IPX(5))))
 
      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5

C...Set particle 6 in 56 CM frame.
      E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))/2.D0/SQRT(GVAL(4))
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999
      PF4(1,IPX(6)) = -PF4(1,IPX(5))
      PF4(2,IPX(6)) = -PF4(2,IPX(5))
      PF4(3,IPX(6)) = -PF4(3,IPX(5))
      PF4(4,IPX(6)) = E6

C...Boost from 56 CM frame to 568 frame.
      CALL GRCW2LB(Q56,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(Q56,PF4(1,IPX(6)),PF4(1,IPX(6)))

C...Boost from 568 CM frame to Lab. frame.
      CALL GRCW2LB(QT2,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(QT2,PF4(1,IPX(6)),PF4(1,IPX(6)))
      CALL GRCW2LB(QT2,PF4(1,IPX(8)),PF4(1,IPX(8)))
      CALL GRCW2LB(QT2,Q56,Q56)

      PQGRC(1,4) = Q56(1)
      PQGRC(2,4) = Q56(2)
      PQGRC(3,4) = Q56(3)
      PQGRC(4,4) = Q56(4)
 
C...Check kinematics.
      DO I = 1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/(2*PI)**4/(32*PI2)**5
     $     *BETA(GVAL(5)/GPRI(3),GVAL(6)/GPRI(3))
     $     *BETA(GVAL(3)/GVAL(5),AMASS2(IPX(7))/GVAL(5))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
     $     *BETA(GVAL(4)/GVAL(6),AMASS2(IPX(8))/GVAL(6))
     $     *BETA(AMASS2(IPX(5))/GVAL(4),AMASS2(IPX(6))/GVAL(4))

      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
